<?php
/**
 * Reusable Content & Text Blocks Plugin by Loomisoft - ls_cb_main Class
 * Copyright (c) 2017 Loomisoft (www.loomisoft.com)
 */

defined( 'LS_CB_PLUGIN' ) or die();

class ls_cb_main {
	static public $plugin_name = 'Reusable Content & Text Blocks by Loomisoft';
	static public $plugin_version = '1.2.1';
	static public $plugin_wp_url = 'https://wordpress.org/plugins/loomisoft-content-blocks/';

	static public $option_name = 'ls_cb_option';
	static public $option_prefix = 'ls_cb_option_';
	static public $form_element_prefix = 'ls-cb-form-item-';
	static public $nonce_name = 'ls_cb_nonce';
	static public $post_type_slug = 'lscontentblock';
	static public $post_type_label = 'Content Block';
	static public $post_type_labels = 'Content Blocks';
	static public $post_type_menu_icon = 'images/ls-wp-menu-icon.png';
	static public $usage_about_page = 'ls_cb_usage_about_page';
	static public $post_type_shortcode = 'ls_content_block';

	static public $plugin_file;
	static public $plugin_path;
	static public $plugin_url;
	static public $plugin_basename;

	static public $circular_block_tracker = array();
	static public $page_title = '';
	static public $content_block_list = array();
	static public $content_block_slug_list = array();
	static public $content_block_list_by_slug = array();
	static public $option_values = array();
	static public $para_list = array(
		'none'                     => 'No Paragraph Tags / Run Shortcodes',
		'no-shortcodes'            => 'No Paragraph Tags / No Shortcodes',
		'paragraphs'               => 'Add Paragraph Tags / Run Shortcodes',
		'paragraphs-no-shortcodes' => 'Add Paragraph Tags / No Shortcodes',
		'full'                     => 'Full Content Filtering'
	);

	static public function start( $plugin_file ) {

		self::$plugin_file     = $plugin_file;
		self::$plugin_path     = plugin_dir_path( self::$plugin_file );
		self::$plugin_url      = plugin_dir_url( self::$plugin_file );
		self::$plugin_basename = plugin_basename( self::$plugin_file );

		if ( $option_values = get_option( self::$option_name ) ) {
			if ( is_array( $option_values ) ) {
				foreach ( $option_values as $option_key => $option_value ) {
					self::$option_values[ $option_key ] = $option_value;
				}
			}
		}
		if ( ! isset( self::$option_values[ 'plugin-version' ] ) ) {
			self::$option_values[ 'plugin-version' ] = '';
		}
		if ( self::get_version_text( self::$plugin_version ) != self::get_version_text( self::$option_values[ 'plugin-version' ] ) ) {
			self::update_option( 'plugin-version', self::$plugin_version );
			self::update_option( 'notice-hide-general-welcome', null );
			self::update_option( 'notice-first-time-general-welcome', null );
			self::update_option( 'notice-impression-count-general-welcome', null );
		}
		update_option( self::$option_name, self::$option_values );

		register_activation_hook( self::$plugin_file, __CLASS__ . '::plugin_activate' );
		register_deactivation_hook( self::$plugin_file, __CLASS__ . '::plugin_deactivate' );
		add_action( 'init', __CLASS__ . '::register_post' );
		add_action( 'widgets_init', __CLASS__ . '::register_widget' );
		add_action( 'wp_head', __CLASS__ . '::do_wp_head' );
		add_action( 'admin_enqueue_scripts', __CLASS__ . '::enqueue_scripts' );
		add_action( 'admin_menu', __CLASS__ . '::admin_menu' );
		add_filter( 'plugin_action_links_' . self::$plugin_basename, __CLASS__ . '::add_action_links' );
		add_action( 'admin_notices', __CLASS__ . '::display_general_welcome' );
		add_filter( 'pre_get_posts', __CLASS__ . '::reorder_list' );
		add_filter( 'manage_' . self::$post_type_slug . '_posts_columns', __CLASS__ . '::set_column_titles' );
		add_action( 'manage_' . self::$post_type_slug . '_posts_custom_column', __CLASS__ . '::set_columns', 10, 2 );
		add_action( 'contextual_help', __CLASS__ . '::contextual_help', 10, 3 );
		add_filter( 'enter_title_here', __CLASS__ . '::change_title_text' );
		add_filter( 'mce_external_plugins', __CLASS__ . '::mce_plugin' );
		add_filter( 'mce_buttons', __CLASS__ . '::mce_button' );
		add_action( 'wp_ajax_ls_cb_mce_popup_form', __CLASS__ . '::mce_popup_form' );
		add_action( 'admin_footer', __CLASS__ . '::mce_popup_form' );
		add_shortcode( self::$post_type_shortcode, __CLASS__ . '::content_block_shortcode' );
		add_filter( 'the_content', __CLASS__ . '::do_the_content' );
		add_action( 'wp_ajax_ls_cb_hide_notice', __CLASS__ . '::hide_notice' );

		return true;
	}

	static public function plugin_activate() {

		flush_rewrite_rules();

	}

	static public function plugin_deactivate() {

		flush_rewrite_rules();

	}

	static public function register_post() {

		$user               = wp_get_current_user();
		$allowed_roles      = array(
			'editor',
			'administrator',
			'author'
		);
		$publicly_queryable = array_intersect( $allowed_roles, $user->roles );

		register_post_type( self::$post_type_slug, array(
			'label'                => __( self::$post_type_labels, 'loomisoft-content-blocks-text-domain' ),
			'labels'               => array(
				'name'               => __( self::$post_type_labels, 'loomisoft-content-blocks-text-domain' ),
				'singular_name'      => __( self::$post_type_label, 'loomisoft-content-blocks-text-domain' ),
				'menu_name'          => __( self::$post_type_labels, 'loomisoft-content-blocks-text-domain' ),
				'name_admin_bar'     => __( self::$post_type_label, 'loomisoft-content-blocks-text-domain' ),
				'all_items'          => __( 'All ' . self::$post_type_labels, 'loomisoft-content-blocks-text-domain' ),
				'add_new'            => __( 'Add New ' . self::$post_type_label, 'loomisoft-content-blocks-text-domain' ),
				'add_new_item'       => __( 'Add New ' . self::$post_type_label, 'loomisoft-content-blocks-text-domain' ),
				'edit_item'          => __( 'Edit ' . self::$post_type_label, 'loomisoft-content-blocks-text-domain' ),
				'new_item'           => __( 'New ' . self::$post_type_label, 'loomisoft-content-blocks-text-domain' ),
				'view_item'          => __( 'View ' . self::$post_type_label, 'loomisoft-content-blocks-text-domain' ),
				'search_items'       => __( 'Search ' . self::$post_type_labels, 'loomisoft-content-blocks-text-domain' ),
				'not_found'          => __( 'No ' . strtolower( self::$post_type_labels ) . ' found', 'loomisoft-content-blocks-text-domain' ),
				'not_found_in_trash' => __( 'No ' . strtolower( self::$post_type_labels ) . ' found in Trash', 'loomisoft-content-blocks-text-domain' ),
				'parent_item_colon'  => __( 'Parent ' . self::$post_type_label, 'loomisoft-content-blocks-text-domain' )
			),
			'public'               => $publicly_queryable,
			'exclude_from_search'  => true,
			'publicly_queryable'   => $publicly_queryable,
			'show_ui'              => true,
			'show_in_nav_menus'    => false,
			'show_in_menu'         => true,
			'show_in_admin_bar'    => true,
			'menu_icon'            => self::$plugin_url . self::$post_type_menu_icon,
			'hierarchical'         => false,
			'supports'             => array(
				'title',
				'editor'
			),
			'register_meta_box_cb' => __CLASS__ . '::meta_box',
			'has_archive'          => false
		) );

		flush_rewrite_rules();

		$args = array(
			'post_type'   => self::$post_type_slug,
			'post_status' => 'any',
			'nopaging'    => true
		);

		$content_blocks = get_posts( $args );

		foreach ( $content_blocks as $content_block ) {
			if ( trim( $content_block->post_title ) == '' ) {
				$new_content_block_values = array(
					'ID'         => $content_block->ID,
					'post_title' => __( 'Content Block', 'loomisoft-content-blocks-text-domain' ) . ' ' . $content_block->ID
				);

				wp_update_post( $new_content_block_values );
			}
		}

		$args = array(
			'post_type'   => self::$post_type_slug,
			'post_status' => 'publish',
			'nopaging'    => true
		);

		$content_blocks = get_posts( $args );

		self::$content_block_list = array();

		foreach ( $content_blocks as $content_block ) {
			self::$content_block_list[ $content_block->ID ]                = $content_block->post_title;
			self::$content_block_slug_list[ $content_block->ID ]           = $content_block->post_name;
			self::$content_block_list_by_slug[ $content_block->post_name ] = $content_block->ID;
		}
	}

	static public function register_widget() {

		register_widget( 'ls_cb_widget' );

	}

	static public function do_wp_head() {

		self::$page_title = get_the_title();

	}

	static public function enqueue_scripts() {

		wp_enqueue_script( 'jquery-ui-dialog', array(
			'jquery',
			'jquery-ui-core'
		) );
		wp_enqueue_style( 'wp-jquery-ui-dialog' );
		wp_enqueue_script( 'ls_cb_admin_script', self::$plugin_url . 'js/admin.js' );
		wp_enqueue_style( 'ls_cb_admin_style', self::$plugin_url . 'css/admin.css' );
		$custom_css = '#adminmenu #menu-posts-' . self::$post_type_slug . ' div.wp-menu-image, #adminmenu #menu-posts-' . self::$post_type_slug . ' div.wp-menu-image img { opacity: 1 !important; }';
		wp_add_inline_style( 'ls_cb_admin_style', $custom_css );

	}

	static public function admin_menu() {

		add_submenu_page( 'edit.php?post_type=' . self::$post_type_slug, __( 'Loomisoft Content Blocks - Usage & About', 'loomisoft-content-blocks-text-domain' ), __( 'Usage & About', 'loomisoft-content-blocks-text-domain' ), 'edit_posts', self::$usage_about_page, __CLASS__ . '::usage_about_page' );

	}

	static public function add_action_links( $links ) {

		$mylinks = array(
			'<a href="' . admin_url( 'edit.php?post_type=' . self::$post_type_slug . '&page=' . self::$usage_about_page ) . '">' . __( 'Usage & About', 'loomisoft-content-blocks-text-domain' ) . '</a>',
		);

		return array_merge( $links, $mylinks );
	}

	static public function reorder_list( $query ) {

		if ( $query->is_admin ) {
			if ( $query->get( 'post_type' ) == self::$post_type_slug ) {
				$query->set( 'orderby', 'post_title' );
				$query->set( 'order', 'ASC' );
			}
		}

		return $query;
	}

	static public function set_column_titles() {

		return array(
			'cb'                                => '<input type="checkbox"/>',
			'title'                             => __( 'Title', 'loomisoft-content-blocks-text-domain' ),
			self::$post_type_slug . 'shortcode' => __( 'Basic Shortcode', 'loomisoft-content-blocks-text-domain' ),
			'date'                              => __( 'Date', 'loomisoft-content-blocks-text-domain' )
		);

	}

	static public function set_columns( $column, $post_id ) {

		switch ( $column ) {
			case self::$post_type_slug . 'shortcode' :

				echo '<input type="text" class="ls_cb_read_only_input" value="' . esc_attr( '[' . self::$post_type_shortcode . ' id="' . $post_id . '"]' ) . '" style="width: 270px !important;"/><br />(' . __( 'Edit post for full list', 'loomisoft-content-blocks-text-domain' ) . ')';
				break;

			default :
				break;
		}

	}

	static public function contextual_help( $contextual_help, $screen_id, $screen ) {
		$current_screen = get_current_screen();

		if ( ( $current_screen->post_type == self::$post_type_slug ) && ( $current_screen->id != self::$post_type_slug . '_page_' . self::$usage_about_page ) ) {
			$screen->remove_help_tabs();

			$content_prefix  = '<div class="ls-cb-admin-info-page-column-content"><div id="ls-cb-admin-branding-right"><a href="http://www.loomisoft.com/" target="' . esc_attr( self::$plugin_basename ) . '"><img src="' . esc_attr( self::$plugin_url ) . 'images/ls-wp-admin-branding.png" /></a></div>';
			$content_postfix = '</div>';

			$help_screens = array(
				'overview' => array(
					'title'   => __( 'Overview', 'loomisoft-content-blocks-text-domain' ),
					'heading' => __( self::$plugin_name, 'loomisoft-content-blocks-text-domain' ) . ' ' . __( 'Version', 'loomisoft-content-blocks-text-domain' ) . ' ' . __( self::$plugin_version, 'loomisoft-content-blocks-text-domain' ),
					'text'    => array(
						__( 'The Reusable Content & Text Blocks plugin by Loomisoft allows you to define modular and repeated blocks of text and other content and place them within pages, posts, sidebars, widgetised areas or anywhere on your site via shortcodes, via the provided widget or via PHP.', 'loomisoft-content-blocks-text-domain' ),
						__( 'The idea behind this plugin is two-fold. The first is to modularise content so you can use the same content in multiple pages, posts and other places. The second is to provide an easy way to add complex custom content within sidebars and widgets.', 'loomisoft-content-blocks-text-domain' ),
						__( 'For full usage information, click below.', 'loomisoft-content-blocks-text-domain' )
					),
					'more'    => array(
						'url'  => admin_url( 'edit.php?post_type=' . self::$post_type_slug . '&page=' . self::$usage_about_page ),
						'text' => __( 'Usage & About', 'loomisoft-content-blocks-text-domain' )
					)
				)
			);

			foreach ( $help_screens as $help_screen_id => $help_screen ) {
				$content = $content_prefix . '<h2>' . esc_html( $help_screen[ 'heading' ] ) . '</h2><p>' . str_replace( '##br##', '<br />', esc_html( implode( '##br####br##', $help_screen[ 'text' ] ) ) );
				if ( ( isset( $help_screen[ 'more' ] ) ) && ( is_array( $help_screen[ 'more' ] ) ) && ( isset( $help_screen[ 'more' ][ 'url' ] ) ) && ( isset( $help_screen[ 'more' ][ 'text' ] ) ) ) {
					$content .= '<br /><br /><a href="' . esc_attr( $help_screen[ 'more' ][ 'url' ] ) . '"';
					if ( ( isset( $help_screen[ 'more' ][ 'target' ] ) ) ) {
						$content .= ' target="' . esc_attr( $help_screen[ 'more' ][ 'target' ] ) . '"';
					}
					$content .= '>' . esc_html( $help_screen[ 'more' ][ 'text' ] ) . '</a>';
				}
				$content .= '</p>' . $content_postfix;

				$screen->add_help_tab( array(
					'id'      => self::$plugin_basename . '-tab-' . $help_screen_id,
					'title'   => $help_screen[ 'title' ],
					'content' => $content
				) );
			}
		}

		return $contextual_help;
	}

	static public function change_title_text( $title ) {
		global $post;

		switch ( $post->post_type ) {
			case '' . self::$post_type_slug:
				$title = __( 'Enter content block name here', 'loomisoft-content-blocks-text-domain' );
				break;
		}

		return $title;
	}

	static public function check_editor() {
		if ( ( current_user_can( 'edit_posts' ) ) || ( current_user_can( 'edit_pages' ) ) ) {
			if ( get_user_option( 'rich_editing' ) == 'true' ) {
				return true;
			}
		}

		return false;
	}

	static public function mce_plugin( $plugin_array ) {
		if ( self::check_editor() ) {
			$plugin_array[ 'ls_cb_button' ] = self::$plugin_url . 'js/editor.js';
		}

		return $plugin_array;
	}

	static public function mce_button( $buttons ) {
		if ( self::check_editor() ) {
			array_push( $buttons, 'ls_cb_button' );
		}

		return $buttons;
	}

	static public function mce_popup_form() {
		if ( self::check_editor() ) {
			echo '<form id="' . esc_attr( self::$option_prefix . 'shortcode_form' ) . '" name="' . esc_attr( self::$option_prefix . 'shortcode_form' ) . '" action="#" method="post">';
			echo '<div class="ls-cb-mce-form-container ls-cb-clearfix">';
			echo '<div id="ls-cb-mce-popup" class="hidden" style="max-width:800px">';
			echo '<div class="ls-cb-mce-field-container ls-cb-clearfix">';
			echo '<div class="ls-cb-mce-label"><label for="' . esc_attr( self::$option_prefix . 'cbid' ) . '">' . esc_html( __( 'Content Block:', 'loomisoft-content-blocks-text-domain' ) ) . '</label></div>';
			echo '<div class="ls-cb-mce-field ls-cb-clearfix"><select id="' . esc_attr( self::$option_prefix . 'cbid' ) . '" name="' . esc_attr( self::$option_prefix . 'cbid' ) . '" class="ls-cb-select-input" message="' . esc_attr( __( 'Content block must be selected', 'loomisoft-content-blocks-text-domain' ) ) . '">';
			echo '<option value="" selected="selected">' . esc_html( __( 'Select Content Block', 'loomisoft-content-blocks-text-domain' ) ) . '</option>';
			foreach ( self::$content_block_list as $content_block_id => $content_block_title ) {
				echo '<option value="' . esc_attr( $content_block_id ) . '" slug="' . esc_attr( self::$content_block_slug_list[ $content_block_id ] ) . '">' . esc_html( $content_block_title ) . '</option>';
			}
			echo '</select></div>';
			echo '</div>';
			echo '<div class="ls-cb-mce-field-container ls-cb-clearfix">';
			echo '<div class="ls-cb-mce-label"><label for="' . esc_attr( self::$option_prefix . 'para' ) . '">' . esc_html( __( 'Content Filtering / Paragraph Tags:', 'loomisoft-content-blocks-text-domain' ) ) . '</label></div>';
			echo '<div class="ls-cb-mce-field ls-cb-clearfix"><select id="' . esc_attr( self::$option_prefix . 'para' ) . '" name="' . esc_attr( self::$option_prefix . 'para' ) . '" class="ls-cb-select-input">';
			foreach ( self::$para_list as $para_key => $para_value ) {
				if ( $para_key == 'none' ) {
					$para_key = '';
				}
				echo '<option value="' . esc_attr( $para_key ) . '"' . ( ( $para_key == '' ) ? ' selected="selected"' : '' ) . '>' . esc_html( __( $para_value, 'loomisoft-content-blocks-text-domain' ) ) . '</option>';
			}
			echo '</select></div>';
			echo '</div>';
			echo '<div class="ls-cb-mce-field-container ls-cb-clearfix">';
			echo '<div class="ls-cb-mce-field ls-cb-clearfix">';
			echo '<div class="ls-cb-mce-field-radio"><input type="radio" id="' . esc_attr( self::$option_prefix . 'use-1' ) . '" name="' . esc_attr( self::$option_prefix . 'use' ) . '" value="id" class="ls-cb-radio-input ' . esc_attr( self::$option_prefix . 'use' ) . '" checked="checked" /></div>';
			echo '<div class="ls-cb-mce-field-radio-label"><label for="' . esc_attr( self::$option_prefix . 'use-0' ) . '">' . esc_html( __( 'By ID', 'loomisoft-content-blocks-text-domain' ) ) . '</label></div>';
			echo '<div class="ls-cb-mce-field-radio"><input type="radio" id="' . esc_attr( self::$option_prefix . 'use-2' ) . '" name="' . esc_attr( self::$option_prefix . 'use' ) . '" value="slug" class="ls-cb-radio-input ' . esc_attr( self::$option_prefix . 'use' ) . '" /></div>';
			echo '<div class="ls-cb-mce-field-radio-label"><label for="' . esc_attr( self::$option_prefix . 'use-1' ) . '">' . esc_html( __( 'By Slug', 'loomisoft-content-blocks-text-domain' ) ) . '</label></div>';
			echo '</div>';
			echo '</div>';
			echo '<div class="ls-cb-mce-field-container ls-cb-clearfix">';
			echo '<div class="ls-cb-mce-field ls-cb-clearfix"><a href="#" id="' . esc_attr( self::$option_prefix . 'insert' ) . '" class="ls-cb-mce-button-blue">' . esc_html( __( 'Insert', 'loomisoft-content-blocks-text-domain' ) ) . '</a></div>';
			echo '</div>';
			echo '</div>';
			echo '</div>';
			echo '</form>';
		}
	}

	static public function meta_box( $post ) {
		add_meta_box( $post->id, __( 'Content Block Usage Codes', 'loomisoft-content-blocks-text-domain' ), __CLASS__ . '::meta_box_screen', self::$post_type_slug, 'normal', 'core' );
	}

	static public function is_edit_page( $new_edit = null ) {
		global $pagenow;
		if ( ! is_admin() ) {
			return false;
		}
		if ( $new_edit == "edit" ) {
			return in_array( $pagenow, array( 'post.php', ) );
		} elseif ( $new_edit == "new" ) {
			return in_array( $pagenow, array( 'post-new.php' ) );
		} else {
			return in_array( $pagenow, array(
				'post.php',
				'post-new.php'
			) );
		}
	}

	static public function meta_box_screen( $post ) {

		$code = array();

		if ( self::is_edit_page( 'edit' ) ) {
			$post_id    = $post->ID;
			$post_slug  = $post->post_name;
			$code[ 0 ]  = '[' . self::$post_type_shortcode . ' id="' . $post_id . '"]';
			$code[ 1 ]  = '[' . self::$post_type_shortcode . ' id="' . $post_id . '" para="no-shortcodes"]';
			$code[ 2 ]  = '[' . self::$post_type_shortcode . ' id="' . $post_id . '" para="paragraphs"]';
			$code[ 3 ]  = '[' . self::$post_type_shortcode . ' id="' . $post_id . '" para="paragraphs-no-shortcodes"]';
			$code[ 4 ]  = '[' . self::$post_type_shortcode . ' id="' . $post_id . '" para="full"]';
			$code[ 5 ]  = '[' . self::$post_type_shortcode . ' slug="' . $post_slug . '"]';
			$code[ 6 ]  = '[' . self::$post_type_shortcode . ' slug="' . $post_slug . '" para="no-shortcodes"]';
			$code[ 7 ]  = '[' . self::$post_type_shortcode . ' slug="' . $post_slug . '" para="paragraphs"]';
			$code[ 8 ]  = '[' . self::$post_type_shortcode . ' slug="' . $post_slug . '" para="paragraphs-no-shortcodes"]';
			$code[ 9 ]  = '[' . self::$post_type_shortcode . ' slug="' . $post_slug . '" para="full"]';
			$code[ 10 ] = '<?php echo ls_content_block_by_id( ' . $post_id . ' ); ?>';
			$code[ 11 ] = '<?php echo ls_content_block_by_id( ' . $post_id . ', \'no-shortcodes\' ); ?>';
			$code[ 12 ] = '<?php echo ls_content_block_by_id( ' . $post_id . ', \'paragraphs\' ); ?>';
			$code[ 13 ] = '<?php echo ls_content_block_by_id( ' . $post_id . ', \'paragraphs-no-shortcodes\' ); ?>';
			$code[ 14 ] = '<?php echo ls_content_block_by_id( ' . $post_id . ', \'full\' ); ?>';
			$code[ 15 ] = '<?php echo ls_content_block_by_slug( \'' . $post_slug . '\' ); ?>';
			$code[ 16 ] = '<?php echo ls_content_block_by_slug( \'' . $post_slug . '\', \'no-shortcodes\' ); ?>';
			$code[ 17 ] = '<?php echo ls_content_block_by_slug( \'' . $post_slug . '\', \'paragraphs\' ); ?>';
			$code[ 18 ] = '<?php echo ls_content_block_by_slug( \'' . $post_slug . '\', \'paragraphs-no-shortcodes\' ); ?>';
			$code[ 19 ] = '<?php echo ls_content_block_by_slug( \'' . $post_slug . '\', \'full\' ); ?>';
		} else {
			for ( $i = 0; $i <= 15; $i ++ ) {
				$code[ $i ] = __( 'New post', 'loomisoft-content-blocks-text-domain' );
			}
		}

		$access            = array(
			__( 'Shortcode', 'loomisoft-content-blocks-text-domain' ),
			__( 'PHP', 'loomisoft-content-blocks-text-domain' )
		);
		$methods           = array(
			__( 'ID', 'loomisoft-content-blocks-text-domain' ),
			__( 'Slug', 'loomisoft-content-blocks-text-domain' )
		);
		$para_descriptions = array(
			__( 'No Paragraph Tags / Run Shortcodes', 'loomisoft-content-blocks-text-domain' ),
			__( 'No Paragraph Tags / No Shortcodes', 'loomisoft-content-blocks-text-domain' ),
			__( 'Add Paragraph Tags / Run Shortcodes', 'loomisoft-content-blocks-text-domain' ),
			__( 'Add Paragraph Tags / No Shortcodes', 'loomisoft-content-blocks-text-domain' ),
			__( 'Full Content Filtering', 'loomisoft-content-blocks-text-domain' )
		);

		for ( $i = 0; $i <= 1; $i ++ ) {
			for ( $j = 0; $j <= 1; $j ++ ) {
				echo '<h1>' . esc_html( __( 'Via', 'loomisoft-content-blocks-text-domain' ) ) . ' ' . esc_html( $access[ $i ] ) . ' / ' . esc_html( __( 'By', 'loomisoft-content-blocks-text-domain' ) ) . ' ' . esc_html( $methods[ $j ] ) . '</h1>';
				echo '<div class="ls_cb_meta">';
				for ( $k = 0; $k <= 4; $k ++ ) {
					$index = ( $i * 10 ) + ( $j * 5 ) + $k;

					echo '<div class="ls_cb_meta_label">
<label for="' . esc_attr( self::$option_prefix . 'access_' . $index ) . '">' . $para_descriptions[ $k ] . '</label>
</div>
<div class="ls_cb_meta_field">
<input type="text" class="ls_cb_read_only_input" id="' . esc_attr( self::$option_prefix . 'access_' . $index ) . '" name="' . esc_attr( self::$option_prefix . 'access_' . $index ) . '" value="' . esc_attr( $code[ $index ] ) . '" />
</div>
';
				}
				echo '</div>';
			}
		}
	}

	static public function get_documetation( $file_name ) {
		$language = strtolower( get_locale() );
		$file     = self::$plugin_path . 'docs/' . $language . '/' . $file_name;
		if ( ! file_exists( $file ) ) {
			$file = self::$plugin_path . 'docs/default/' . $file_name;
		}
		$contents = '';
		if ( file_exists( $file ) ) {
			$raw_contents = file_get_contents( $file );
			if ( $raw_contents !== false ) {
				if ( preg_match( '/<body>((.|\n)*?)<\/body>/', $raw_contents, $matches ) === 1 ) {
					$contents = $matches[ 1 ];
					$contents = str_replace( '<li>', '<li style="padding-left: 30px; position: relative;"><span style="display: block; position: absolute; left: 10px;">•</span>', $contents );
					$contents = str_replace( '<!--[', '', $contents );
					$contents = str_replace( ']-->', '', $contents );
					$contents = str_replace( 'href="{a}', 'href="' . get_admin_url(), $contents );
					$contents = str_replace( 'src="../', 'src="' . self::$plugin_url, $contents );
				}
			}
		}

		return $contents;
	}

	static public function usage_about_page() {
		echo '<div class="wrap"><h1>' . esc_html( __( 'Usage & About', 'loomisoft-content-blocks-text-domain' ) ) . ' - ' . esc_html( __( self::$plugin_name, 'loomisoft-content-blocks-text-domain' ) ) . ' ' . esc_html( __( 'Version', 'loomisoft-content-blocks-text-domain' ) ) . ' ' . esc_html( __( self::$plugin_version, 'loomisoft-content-blocks-text-domain' ) ) . '</h1>';
		echo '<div class="ls-cb-admin-info-page-row ls-cb-clearfix">';
		echo '<div class="ls-cb-admin-info-page-column-half">';
		echo '<div class="ls-cb-admin-info-page-column-inner">';
		echo '<div class="postbox">';
		echo '<div class="inside">';
		echo self::get_documetation( 'usage-about.htm' );
		echo '</div>';
		echo '</div>';
		echo '</div>';
		echo '</div>';
		echo '<div class="ls-cb-admin-info-page-column-half">';
		echo '<div class="ls-cb-admin-info-page-column-inner">';
		echo '<div class="postbox">';
		echo '<div class="inside">';
		echo self::get_documetation( 'questions-comments.htm' );
		echo '</div>';
		echo '</div>';
		echo '<div class="postbox">';
		echo '<div class="inside">';
		echo self::get_documetation( 'rate-this-plugin.htm' );
		echo '</div>';
		echo '</div>';
		echo '<div class="postbox">';
		echo '<div class="inside">';
		echo self::get_documetation( 'make-a-financial-contribution.htm' );
		echo '</div>';
		echo '</div>';
		echo '</div>';
		echo '</div>';
	}

	static public function do_the_content( $content ) {

		if ( get_post_type() == self::$post_type_slug ) {
			if ( class_exists( 'Vc_Base' ) ) {
				$vc = new Vc_Base;
				$vc->addFrontCss();
			}
		}

		return $content;
	}

	static public function get_content( $id, $para = false ) {

		if ( in_array( $id, self::$circular_block_tracker ) ) {
			return '';
		}

		self::$circular_block_tracker[] = $id;

		$post_content = '';

		switch ( $para ) {

			case 'full':

				$args = array(
					'p'           => $id,
					'post_type'   => self::$post_type_slug,
					'post_status' => 'publish',
					'nopaging'    => true
				);

				if ( class_exists( 'FLBuilder' ) ) {
					FLBuilder::render_query( $args );
				} else {
					$the_query = new WP_Query( $args );

					if ( $the_query->have_posts() ) {
						while ( $the_query->have_posts() ) {
							$the_query->the_post();

							$post_content = apply_filters( 'the_content', get_the_content() );

							break;
						}
					}
					wp_reset_postdata();
				}

				break;

			default:

				$args       = array(
					'p'           => $id,
					'post_type'   => self::$post_type_slug,
					'post_status' => 'publish',
					'nopaging'    => true
				);
				$block_post = get_posts( $args );

				if ( count( $block_post ) > 0 ) {
					switch ( $para ) {

						case 'paragraphs':
							$post_content = do_shortcode( wpautop( $block_post[ 0 ]->post_content ) );
							break;

						case 'paragraphs-no-shortcodes':
							$post_content = wpautop( $block_post[ 0 ]->post_content );
							break;

						case 'no-shortcodes':
							$post_content = $block_post[ 0 ]->post_content;
							break;

						default:
							$post_content = do_shortcode( $block_post[ 0 ]->post_content );
							break;

					}
				}

				break;
		}

		foreach ( array_keys( self::$circular_block_tracker, $id ) as $key ) {
			unset( self::$circular_block_tracker[ $key ] );
		}

		return $post_content;
	}

	static public function get_block_by_id( $id = false, $para = false ) {
		$html = '';

		$id   = self::get_clean_id( $id );
		$para = self::get_clean_para( $para );

		if ( $id !== false ) {
			$html = self::get_content( $id, $para );
		}

		return $html;
	}

	static public function get_block_by_slug( $slug = false, $para = false ) {
		$html = '';

		$slug = self::get_clean_slug( $slug );
		$para = self::get_clean_para( $para );

		if ( $slug !== false ) {
			$html = self::get_content( self::$content_block_list_by_slug[ $slug ], $para );
		}

		return $html;
	}

	static public function get_clean_id( $id = false ) {
		if ( preg_match( '/^((0{1})|([1-9]{1})|([1-9]{1}[0-9]*))$/', trim( strval( $id ) ) ) === 1 ) {
			if ( isset( self::$content_block_list[ $id ] ) ) {
				return intval( $id );
			}
		}

		return false;
	}

	static public function get_clean_slug( $slug = '' ) {
		$slug = trim( strval( $slug ) );
		if ( $slug != '' ) {
			if ( isset( self::$content_block_list_by_slug[ $slug ] ) ) {
				return $slug;
			}
		}

		return false;
	}

	static public function get_clean_para( $para = '' ) {
		if ( $para === false ) {
			return '';
		} elseif ( $para === true ) {
			return 'full';
		} else {
			$para = strtolower( trim( strval( $para ) ) );
			if ( in_array( $para, array(
				'',
				'no-shortcodes',
				'yes',
				'full',
				'paragraphs',
				'paragraphs-no-shortcodes'
			) ) ) {
				if ( $para == 'yes' ) {
					$para = 'full';
				}

				return $para;
			}
		}

		return '';
	}

	static public function content_block_shortcode( $atts ) {

		$html = '';

		$para = '';

		if ( isset( $atts[ 'para' ] ) ) {
			$para = $atts[ 'para' ];
		}

		if ( isset( $atts[ 'id' ] ) ) {
			$html = self::get_block_by_id( $atts[ 'id' ], $para );
		} elseif ( isset( $atts[ 'slug' ] ) ) {
			$html = self::get_block_by_slug( $atts[ 'slug' ], $para );
		} elseif ( isset( $atts[ 'datetime' ] ) ) {
			if ( $datetime = date( $atts[ 'datetime' ] ) ) {
				$html = esc_html( $datetime );
			}
		} elseif ( isset( $atts[ 'info' ] ) ) {
			$info = strtolower( trim( strval( $atts[ 'info' ] ) ) );
			switch ( $info ) {
				case 'site-title':
					$html = esc_html( get_bloginfo( 'name' ) );
					break;
				case 'page-title':
					$html = esc_html( self::$page_title );
					break;
			}
		}

		return $html;
	}

	static public function get_version_text( $version ) {
		$version_text = sprintf( '%020d', 0 );
		if ( preg_match( '/^(([0-9]{1})|([1-9]{1}[0-9]*))(\.(([0-9]{1})|([1-9]{1}[0-9]*))(\.(([0-9]{1})|([1-9]{1}[0-9]*))(\.(([0-9]{1})|([1-9]{1}[0-9]*))){0,1}){0,1}){0,1}$/', $version, $matches ) === 1 ) {
			if ( ! is_array( $matches ) ) {
				$matches = array();
			}
			$version_text   = '';
			$version_digits = array();
			$digits         = array(
				1,
				5,
				9,
				13
			);
			foreach ( $digits as $digit ) {
				if ( isset( $matches[ $digit ] ) ) {
					$version_digits[] = $matches[ $digit ];
				} else {
					$version_digits[] = '0';
				}
			}
			foreach ( $version_digits as $key => $version_digit ) {
				$version_text .= sprintf( '%05d', substr( $version_digit, 0, 5 ) );
			}
		}

		return $version_text;
	}

	static public function get_option( $option_name ) {
		if ( isset( self::$option_values[ $option_name ] ) ) {
			return self::$option_values[ $option_name ];
		}

		return false;
	}

	static public function update_option( $option_key, $option_value ) {
		if ( is_null( $option_value ) ) {
			unset( self::$option_values[ $option_key ] );
		} else {
			self::$option_values[ $option_key ] = $option_value;
		}
		update_option( self::$option_name, self::$option_values );
	}

	static public function hide_notice() {
		if ( isset( $_POST[ 'ls-cb-notice-hide' ] ) ) {
			self::update_option( 'notice-hide-' . trim( $_POST[ 'ls-cb-notice-hide' ] ), true );
			echo 'ok';
		}

		wp_die();
	}

	static public function display_general_welcome() {
		$current_screen = get_current_screen();

		$first_time       = self::get_option( 'notice-first-time-general-welcome' );
		$impression_count = self::get_option( 'notice-impression-count-general-welcome' );

		if ( ( $first_time === false ) || ( preg_match( '/^((0{1})|([1-9]{1})|([1-9]{1}[0-9]*))$/', trim( strval( $first_time ) ) ) !== 1 ) || ( $impression_count === false ) || ( preg_match( '/^((0{1})|([1-9]{1})|([1-9]{1}[0-9]*))$/', trim( strval( $impression_count ) ) ) !== 1 ) ) {
			$first_time = time();
			self::update_option( 'notice-first-time-general-welcome', $first_time );
			$impression_count = 0;
			self::update_option( 'notice-impression-count-general-welcome', $impression_count );
		}

		if ( ( time() - $first_time < 604800 ) && ( $impression_count < 200 ) ) { // Stop displaying after 7 days or 200 times
			if ( ( $current_screen->post_type == self::$post_type_slug ) && ( ! self::get_option( 'notice-hide-general-welcome' ) ) ) {
				$heading = esc_html( __( 'Welcome to version ' . self::$plugin_version . ' of ' . self::$plugin_name, 'loomisoft-content-blocks-text-domain' ) );
				if ( $current_screen->id == self::$post_type_slug . '_page_' . self::$usage_about_page ) {
					$message = esc_html( __( 'If you are new to the plugin, full usage instructions can be found below.', 'loomisoft-content-blocks-text-domain' ) ) . '<br /><br />' . esc_html( __( 'For existing users, this update features improvements to usability by providing an easy access shortcode generator dialogue box so you can easily and quickly insert shortcodes directly into your WordPress post/page editor, meaning you don\'t have to leave your edit screen to get the code you need. We hope you find this and the other recent improvements useful.', 'loomisoft-content-blocks-text-domain' ) ) . '<br /><br /><em>' . esc_html( __( 'Enjoy!', 'loomisoft-content-blocks-text-domain' ) ) . '</em>';
				} else {
					$message = esc_html( __( 'If you are new to the plugin, full usage instructions can be found on the ', 'loomisoft-content-blocks-text-domain' ) ) . ' <a href="' . admin_url( 'edit.php?post_type=' . self::$post_type_slug . '&page=' . self::$usage_about_page ) . '">' . esc_html( __( 'Usage & About', 'loomisoft-content-blocks-text-domain' ) ) . '</a> ' . esc_html( __( 'page.', 'loomisoft-content-blocks-text-domain' ) ) . '<br /><br />' . esc_html( __( 'For existing users, this update features improvements to usability by providing an easy access shortcode generator dialogue box so you can easily and quickly insert shortcodes directly into your WordPress post/page editor, meaning you don\'t have to leave your edit screen to get the code you need. We hope you find this and the other recent improvements useful.', 'loomisoft-content-blocks-text-domain' ) ) . '<br /><br /><em>' . esc_html( __( 'Enjoy!', 'loomisoft-content-blocks-text-domain' ) ) . '</em>';
				}
				self::display_notice( 'general-welcome', $heading, $message, true );
				self::update_option( 'notice-impression-count-general-welcome', $impression_count + 1 );
			}
		}
	}

	static public function display_notice( $notice_id, $heading, $message, $hideable ) {
		if ( ! self::get_option( 'notice-hide-' . $notice_id ) ) {
			echo '<div id="ls-cb-notice-' . esc_attr( $notice_id ) . '" class="ls-cb-notice notice is-dismissible"><div class="ls-cb-admin-info-page-column-content ls-cb-clearfix"><div id="ls-cb-admin-branding-right"><a href="http://www.loomisoft.com/" target="' . esc_attr( self::$plugin_basename ) . '"><img src="' . esc_attr( self::$plugin_url ) . 'images/ls-wp-admin-branding.png" /></a></div><h2>' . $heading . '</h2><p>' . $message . '</p>';
			if ( $hideable ) {
				echo '<p class="ls-cb-notice-hide-container"><a id="ls-cb-notice-hide-' . esc_attr( $notice_id ) . '" class="ls-cb-notice-hide-link" href="#">' . esc_html( __( 'Hide this notice', 'loomisoft-content-blocks-text-domain' ) ) . '</a></p>';
			}
			echo '</div></div>';
		}
	}
}
